@@ -9,8 +9,20 @@ use B::Hooks::Parser qw();
 use Carp qw(croak);
 use Exporter::Shiny qw(import);
 
+BEGIN {
+	eval {
+		require List::MoreUtils;
+		'List::MoreUtils'->import('uniq');
+		1;
+	}
+	or *uniq = sub {
+		my %already;
+		grep !$already{$_}++, @_;
+	};
+};
+
 our $AUTHORITY   = 'cpan:TOBYINK';
-our $VERSION     = '0.000_02';
+our $VERSION     = '0.000_03';
 our %EXPORT_TAGS = ( setup => [qw(import)] );
 our @INIT;
 
@@ -23,53 +35,86 @@ sub _generate_import
 	(my $nominal_file = $caller) =~ s(::)(/)g;
 	$INC{"$nominal_file\.pm"} ||= __FILE__;
 	
-	my %inject;
-	for my $var (sort keys %$args)
+	my (%inject, @vars, @tags);
+	for (sort keys %$args)
 	{
-		unless ($var =~ /\A[\$\@\%][^\W0-9]\w*\z/)
-		{
-			croak("Not a legal name for a lexical variable: '$var'");
-		}
-		
-		my $value = $args->{$var};
-		
-		if ($var =~ /\A[\@\%]/ and defined($value) and not ref($value))
-		{
-			croak("Cannot initialize $var from a scalar value");
-		}
-		
-		if (!ref($value))
-		{
-			$inject{$var} = sprintf(
-				'my(%s)=(%s);',
-				$var,
-				defined($value) ? B::perlstring($value) : '',
-			);
-		}
-		elsif (ref($value) eq q(CODE))
-		{
-			push @INIT, $value;
-			$inject{$var} = sprintf(
-				'my(%s);$%s::INIT[%d]->(\\%s, %s);',
-				$var,
-				__PACKAGE__,
-				$#INIT,
-				$var,
-				B::perlstring($var),
-			);
-		}
-		else
-		{
-			croak("Cannot initialize $var from reference of type " . ref($value));
-		}
+		/\A[\$\@\%][^\W0-9]\w*\z/ ? push(@vars, $_) :
+		/\A[:-](\w*)\z/           ? push(@tags, ":$1") :
+		croak("Not a legal name for a lexical variable or tag: '$_'");
 	}
 	
-	return sub
+	$inject{$_} = $me->_setup_lexical_variable($args, $_) for @vars;
+	$inject{$_} = $me->_setup_tag($args, $_) for @tags;
+	$inject{":all"} ||= [sort @vars];
+	
+	return sub { $me->_handle_inject(\%inject, @_) };
+}
+
+sub _handle_inject
+{
+	my $me      = shift;
+	my $inject  = shift;
+	my $package = shift;
+	
+	# Handle tag expansion
+	@_ = @{ $inject->{':default'} or $inject->{':all'} or [] } unless @_;
+	@_ = uniq map {
+		/\A[:-](\w*)\z/
+			? @{ $inject->{":$1"} or croak("Tag $1 not exported by $package") }
+			: $_
+	} @_;
+	
+	my @code = map {
+		$inject->{$_} or croak("Variable $_ not exported by $package")
+	} @_;
+	B::Hooks::Parser::inject(join(';', '', @code));
+}
+
+sub _setup_lexical_variable
+{
+	my $me = shift;
+	my ($args, $var) = @_;
+	my $value = $args->{$var};
+	
+	if ($var =~ /\A[\@\%]/ and defined($value) and not ref($value))
 	{
-		shift;
-		@_ = sort keys(%inject) unless @_;
-		B::Hooks::Parser::inject(join ';', '', map $inject{$_}, @_);
-	};
+		croak("Cannot initialize $var from a scalar value");
+	}
+	
+	if (!ref($value))
+	{
+		return sprintf(
+			'my(%s)=(%s);',
+			$var,
+			defined($value) ? B::perlstring($value) : '',
+		);
+	}
+	
+	if (ref($value) eq q(CODE))
+	{
+		push @INIT, $value;
+		return sprintf(
+			'my(%s);$%s::INIT[%d]->(\\%s, %s);',
+			$var,
+			__PACKAGE__,
+			$#INIT,
+			$var,
+			B::perlstring($var),
+		);
+	}
+	
+	croak("Cannot setup variable $var from reference of type " . ref($value));
+}
+
+sub _setup_tag
+{
+	my $me = shift;
+	my ($args, $var) = @_;
+	my $value = $args->{$var};
+	
+	return $value if ref($value) eq q(ARRAY);
+	
+	croak("Cannot setup tag $var from reference of type " . ref($value));
 }
 
 1;
@@ -135,6 +180,24 @@ of arrays and hashes, providing C<undef> in the setup hashref will
 initialize them to empty, but otherwise you must initialize them with
 an initializer coderef; not a non-reference scalar value.
 
+L<Exporter>-style "tags" may also be defined:
+
+	BEGIN {
+		package MyVars;
+		use Exporter::LexicalVars -setup => {
+			'$pi'     => sub { ... },
+			'$e'      => sub { ... },
+			':maths'  => [qw( $pi $e )],
+		};
+	};
+	
+	use MyVars qw( :maths );
+	use MyVars -maths;           # this works too
+
+A tag called C<< :all >> will be automatically set up for you. If your
+exporter is called without any arguments, then it will export a tag called
+C<< :default >> if it exists, or C<< :all >> otherwise.
+
 =head1 BUGS
 
 Please report any bugs to
